#!/usr/bin/python3
# -*- coding: utf-8 -*-
currency_symbols = {'AUD': 'A$',
                    'BGN': 'BGN',
                    'BRL': 'R$',
                    'CAD': 'CA$',
                    'CHF': 'CHF',
                    'CNY': 'CN¥',
                    'CZK': 'CZK',
                    'DKK': 'DKK',
                    'EUR': '€',
                    'GBP': '£',
                    'HKD': 'HK$',
                    'HRK': 'HRK',
                    'HUF': 'HUF',
                    'IDR': 'IDR',
                    'ILS': '₪',
                    'INR': '₹',
                    'ISK': 'ISK',
                    'JPY': '¥',
                    'KRW': '₩',
                    'MXN': 'MX$',
                    'MYR': 'MYR',
                    'NOK': 'NOK',
                    'NZD': 'NZ$',
                    'PHP': 'PHP',
                    'PLN': 'PLN',
                    'RON': 'RON',
                    'SEK': 'SEK',
                    'SGD': 'SGD',
                    'THB': 'THB',
                    'TRY': 'TRY',
                    'USD': '$',
                    'ZAR': 'ZAR',
                    }
currency_names = {'AUD': 'Australian Dollar',
                  'BGN': 'Bulgarian Lev',
                  'BRL': 'Brazilian Real',
                  'CAD': 'Canadian Dollar',
                  'CHF': 'Swiss Franc',
                  'CNY': 'Chinese Yuan',
                  'CZK': 'Czech Koruna',
                  'DKK': 'Danish Krone',
                  'EUR': 'Euro',
                  'GBP': 'British Pound',
                  'HKD': 'Hong Kong Dollar',
                  'HRK': 'Croatian Kuna',
                  'HUF': 'Hungarian Forint',
                  'IDR': 'Indonesian Rupiah',
                  'ILS': 'Israeli New Shekel',
                  'INR': 'Indian Rupee',
                  'ISK': 'Icelandic Króna',
                  'JPY': 'Japanese Yen',
                  'KRW': 'South Korean Won',
                  'MXN': 'Mexican Peso',
                  'MYR': 'Malaysian Ringgit',
                  'NOK': 'Norwegian Krone',
                  'NZD': 'New Zealand Dollar',
                  'PHP': 'Philippine Piso',
                  'PLN': 'Polish Zloty',
                  'RON': 'Romanian Leu',
                  'SEK': 'Swedish Krona',
                  'SGD': 'Singapore Dollar',
                  'THB': 'Thai Baht',
                  'TRY': 'Turkish Lira',
                  'USD': 'US Dollar',
                  'ZAR': 'South African Rand',
                  }
